@@ -71,7 +71,7 @@ class Backend():
         return dirname
 
     def get_target_private_dir(self, target):
-        dirname = os.path.join(self.get_target_dir(target), target.get_basename() + '.dir')
+        dirname = os.path.join(self.get_target_dir(target), target.get_basename() + target.type_suffix())
         os.makedirs(os.path.join(self.environment.get_build_dir(), dirname), exist_ok=True)
         return dirname
 
@@ -170,7 +170,13 @@ class Backend():
             osrc_base = osrc.fname
             if not self.source_suffix_in_objs:
                 osrc_base = '.'.join(osrc.split('.')[:-1])
-            objbase = os.path.join(osrc.subdir, osrc.fname).replace('/', '_').replace('\\', '_')
+            # If extracting in a subproject, the subproject
+            # name gets duplicated in the file name.
+            pathsegs = osrc.subdir.split(os.sep)
+            if pathsegs[0] == 'subprojects':
+                pathsegs = pathsegs[2:]
+            fixedpath = os.sep.join(pathsegs)
+            objbase = osrc.fname.replace('/', '_').replace('\\', '_')
             objname = os.path.join(proj_dir_to_build_root,
                                    targetdir, os.path.basename(objbase) + suffix)
             result.append(objname)
@@ -196,9 +202,9 @@ class Backend():
         commands += compiler.get_always_args()
         commands += self.build.get_global_args(compiler)
         commands += self.environment.coredata.external_args[compiler.get_language()]
-        commands += target.get_extra_args(compiler.get_language())
         if self.environment.coredata.buildtype != 'plain':
             commands += compiler.get_std_warn_args()
+        commands += target.get_extra_args(compiler.get_language())
         commands += compiler.get_buildtype_args(self.environment.coredata.buildtype)
         if self.environment.coredata.coverage:
             commands += compiler.get_coverage_args()
