@@ -17,7 +17,7 @@ import environment
 import dependencies
 import mlog
 import copy, os
-from mesonlib import File
+from mesonlib import File, flatten
 
 known_basic_kwargs = {'install' : True,
                       'c_pch' : True,
@@ -25,6 +25,7 @@ known_basic_kwargs = {'install' : True,
                       'c_args' : True,
                       'cpp_args' : True,
                       'cs_args' : True,
+                      'vala_args' : True,
                       'link_args' : True,
                       'link_depends': True,
                       'link_with' : True,
@@ -39,7 +40,7 @@ known_basic_kwargs = {'install' : True,
                       'sources' : True,
                       'objects' : True,
                       'native' : True,
-                      }
+                     }
 
 known_shlib_kwargs = known_basic_kwargs.copy()
 known_shlib_kwargs.update({'version' : True,
@@ -113,21 +114,12 @@ class Build:
     def get_data(self):
         return self.data
 
-    def get_configure_files(self):
-        return self.configure_files
-
     def get_install_subdirs(self):
         return self.install_dirs
 
     def get_global_args(self, compiler):
         return self.global_args.get(compiler.get_language(), [])
 
-    def get_external_args(self, compiler):
-        return self.external_args.get(compiler.get_language(), [])
-
-    def get_external_link_args(self, compiler):
-        return self.external_link_args.get(compiler.get_language(), [])
-
 class IncludeDirs():
     def __init__(self, curdir, dirs):
         self.curdir = curdir
@@ -308,6 +300,10 @@ class BuildTarget():
         if not isinstance(cslist, list):
             cslist = [cslist]
         self.add_compiler_args('cs', cslist)
+        valalist = kwargs.get('vala_args', [])
+        if not isinstance(valalist, list):
+            valalist = [valalist]
+        self.add_compiler_args('vala', valalist)
         self.link_args = kwargs.get('link_args', [])
         if not isinstance(self.link_args, list):
             self.link_args = [self.link_args]
@@ -320,10 +316,6 @@ class BuildTarget():
         for i in self.link_depends:
             if not isinstance(i, str):
                 raise InvalidArguments('Link_depends arguments must be strings.')
-        if 'version' in kwargs:
-            self.set_version(kwargs['version'])
-        if 'soversion' in kwargs:
-            self.set_soversion(kwargs['soversion'])
         inclist = kwargs.get('include_directories', [])
         if not isinstance(inclist, list):
             inclist = [inclist]
@@ -421,18 +413,20 @@ class BuildTarget():
         for dep in deps:
             if hasattr(dep, 'held_object'):
                 dep = dep.held_object
-            if not isinstance(dep, dependencies.Dependency):
-                raise InvalidArguments('Argument is not an external dependency')
-            self.external_deps.append(dep)
-            if isinstance(dep, dependencies.Dependency):
+            if isinstance(dep, dependencies.InternalDependency):
+                self.process_sourcelist(dep.sources)
+                self.add_include_dirs(dep.include_directories)
+                for l in dep.libraries:
+                    self.link(l)
+            elif isinstance(dep, dependencies.Dependency):
+                self.external_deps.append(dep)
                 self.process_sourcelist(dep.get_sources())
+            else:
+                raise InvalidArguments('Argument is not an external dependency')
 
     def get_external_deps(self):
         return self.external_deps
 
-    def add_dep(self, args):
-        [self.add_external_dep(dep) for dep in args]
-
     def link(self, target):
         if not isinstance(target, list):
             target = [target]
@@ -482,8 +476,9 @@ class BuildTarget():
         self.include_dirs += ids
 
     def add_compiler_args(self, language, args):
+        args = flatten(args)
         for a in args:
-            if not isinstance(a, str):
+            if not isinstance(a, (str, File)):
                 raise InvalidArguments('A non-string passed to compiler args.')
         if language in self.extra_args:
             self.extra_args[language] += args
@@ -590,6 +585,8 @@ class Executable(BuildTarget):
         else:
             self.filename = self.name
 
+    def type_suffix(self):
+        return "@exe"
 
 class StaticLibrary(BuildTarget):
     def __init__(self, name, subdir, is_cross, sources, objects, environment, kwargs):
@@ -606,6 +603,9 @@ class StaticLibrary(BuildTarget):
     def get_osx_filename(self):
         return self.get_filename()
 
+    def type_suffix(self):
+        return "@sta"
+
 class SharedLibrary(BuildTarget):
     def __init__(self, name, subdir, is_cross, sources, objects, environment, kwargs):
         self.version = None
@@ -619,6 +619,13 @@ class SharedLibrary(BuildTarget):
             self.suffix = environment.get_shared_lib_suffix()
         self.importsuffix = environment.get_import_lib_suffix()
 
+    def process_kwargs(self, kwargs, environment):
+        super().process_kwargs(kwargs, environment)
+        if 'version' in kwargs:
+            self.set_version(kwargs['version'])
+        if 'soversion' in kwargs:
+            self.set_soversion(kwargs['soversion'])
+
     def check_unknown_kwargs(self, kwargs):
         self.check_unknown_kwargs_int(kwargs, known_shlib_kwargs)
 
@@ -646,7 +653,6 @@ class SharedLibrary(BuildTarget):
 
     def set_version(self, version):
         if not isinstance(version, str):
-            print(version)
             raise InvalidArguments('Shared library version is not a string.')
         self.version = version
 
@@ -665,6 +671,9 @@ class SharedLibrary(BuildTarget):
             aliases.append(self.get_shbase())
         return aliases
 
+    def type_suffix(self):
+        return "@sha"
+
 class CustomTarget:
     known_kwargs = {'input' : True,
                     'output' : True,
@@ -787,6 +796,9 @@ class CustomTarget:
     def get_generated_sources(self):
         return []
 
+    def type_suffix(self):
+        return "@cus"
+
 class RunTarget:
     def __init__(self, name, command, args, subdir):
         self.name = name
@@ -815,6 +827,9 @@ class RunTarget:
     def get_filename(self):
         return self.name
 
+    def type_suffix(self):
+        return "@run"
+
 class Jar(BuildTarget):
     def __init__(self, name, subdir, is_cross, sources, objects, environment, kwargs):
         super().__init__(name, subdir, is_cross, sources, objects, environment, kwargs);
@@ -826,6 +841,9 @@ class Jar(BuildTarget):
     def get_main_class(self):
         return self.main_class
 
+    def type_suffix(self):
+        return "@jar"
+
 class ConfigureFile():
 
     def __init__(self, subdir, sourcename, targetname, configuration_data):
@@ -837,9 +855,6 @@ class ConfigureFile():
     def get_configuration_data(self):
         return self.configuration_data
 
-    def get_sources(self):
-        return self.sources
-
     def get_subdir(self):
         return self.subdir
 
