@@ -15,6 +15,7 @@
 import subprocess, os.path
 import tempfile
 import mesonlib
+import mlog
 from coredata import MesonException
 
 """This file contains the data files of all compilers Meson knows
@@ -46,15 +47,14 @@ def is_object(fname):
     return suffix in obj_suffixes
 
 gnulike_buildtype_args = {'plain' : [],
-                           'debug' : ['-g'],
-                           'debugoptimized' : ['-O2', '-g'],
-                           'release' : ['-O3'],
-                           }
+                          'debug' : ['-g'],
+                          'debugoptimized' : ['-O2', '-g'],
+                          'release' : ['-O3']}
 
 msvc_buildtype_args = {'plain' : [],
-                        'debug' : ["/MDd", "/Zi", "/Ob0", "/Od", "/RTC1"],
-                        'debugoptimized' : ["/MD", "/Zi", "/O2", "/Ob1", "/D"],
-                        'release' : ["/MD", "/O2", "/Ob2"]}
+                       'debug' : ["/MDd", "/Zi", "/Ob0", "/Od", "/RTC1"],
+                       'debugoptimized' : ["/MD", "/Zi", "/O2", "/Ob1", "/D"],
+                       'release' : ["/MD", "/O2", "/Ob2"]}
 
 gnulike_buildtype_linker_args = {}
 
@@ -63,18 +63,18 @@ if mesonlib.is_osx():
                                           'debug' : [],
                                           'debugoptimized' : [],
                                           'release' : [],
-                                          })
+                                         })
 else:
     gnulike_buildtype_linker_args.update({'plain' : [],
                                           'debug' : [],
                                           'debugoptimized' : [],
                                           'release' : ['-Wl,-O1'],
-                                          })
+                                         })
 
 msvc_buildtype_linker_args = {'plain' : [],
-                               'debug' : [],
-                               'debugoptimized' : [],
-                               'release' : []}
+                              'debug' : [],
+                              'debugoptimized' : [],
+                              'release' : []}
 
 java_buildtype_args = {'plain' : [],
                        'debug' : ['-g'],
@@ -82,9 +82,9 @@ java_buildtype_args = {'plain' : [],
                        'release' : []}
 
 rust_buildtype_args = {'plain' : [],
-                        'debug' : ['-g'],
-                        'debugoptimized' : ['-g', '--opt-level', '2'],
-                        'release' : ['--opt-level', '3']}
+                       'debug' : ['-g'],
+                       'debugoptimized' : ['-g', '--opt-level', '2'],
+                       'release' : ['--opt-level', '3']}
 
 mono_buildtype_args = {'plain' : [],
                        'debug' : ['-debug'],
@@ -251,6 +251,7 @@ int someSymbolHereJustForFun;
         return self.compiles(templ % hname)
 
     def compiles(self, code):
+        mlog.debug('Running compile test:\n\n', code)
         suflen = len(self.default_suffix)
         (fd, srcname) = tempfile.mkstemp(suffix='.'+self.default_suffix)
         os.close(fd)
@@ -260,8 +261,12 @@ int someSymbolHereJustForFun;
         commands = self.get_exelist()
         commands += self.get_compile_only_args()
         commands.append(srcname)
-        p = subprocess.Popen(commands, cwd=os.path.split(srcname)[0], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
-        p.communicate()
+        p = subprocess.Popen(commands, cwd=os.path.split(srcname)[0], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        (stde, stdo) = p.communicate()
+        stde = stde.decode()
+        stdo = stdo.decode()
+        mlog.debug('Compiler stdout:\n', stdo)
+        mlog.debug('Compiler stderr:\n', stde)
         os.remove(srcname)
         try:
             trial = srcname[:-suflen] + 'o'
@@ -275,6 +280,7 @@ int someSymbolHereJustForFun;
         return p.returncode == 0
 
     def run(self, code):
+        mlog.debug('Running code:\n\n', code)
         if self.is_cross and self.exe_wrapper is None:
             raise CrossNoRunException('Can not run test applications in this cross environment.')
         (fd, srcname) = tempfile.mkstemp(suffix='.'+self.default_suffix)
@@ -286,8 +292,12 @@ int someSymbolHereJustForFun;
         commands = self.get_exelist()
         commands.append(srcname)
         commands += self.get_output_args(exename)
-        p = subprocess.Popen(commands, cwd=os.path.split(srcname)[0], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
-        p.communicate()
+        p = subprocess.Popen(commands, cwd=os.path.split(srcname)[0], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        (stdo, stde) = p.communicate()
+        stde = stde.decode()
+        stdo = stdo.decode()
+        mlog.debug('Compiler stdout:\n', stdo)
+        mlog.debug('Compiler stderr:\n', stde)
         os.remove(srcname)
         if p.returncode != 0:
             return RunResult(False)
@@ -297,8 +307,12 @@ int someSymbolHereJustForFun;
             cmdlist = exename
         pe = subprocess.Popen(cmdlist, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
         (so, se) = pe.communicate()
+        so = so.decode()
+        se = se.decode()
+        mlog.debug('Program stdout:\n', so)
+        mlog.debug('Program stderr:\n', se)
         os.remove(exename)
-        return RunResult(True, pe.returncode, so.decode(), se.decode())
+        return RunResult(True, pe.returncode, so, se)
 
     def sizeof(self, element, prefix, env):
         templ = '''#include<stdio.h>
@@ -795,6 +809,9 @@ class ValaCompiler():
         self.id = 'unknown'
         self.language = 'vala'
 
+    def name_string(self):
+        return ' '.join(self.exelist)
+
     def needs_static_linker(self):
         return False # Because compiles into C.
 
@@ -820,8 +837,9 @@ class ValaCompiler():
         if pc.returncode != 0:
             raise EnvironmentException('Vala compiler %s can not compile programs.' % self.name_string())
 
-    def can_compile(self, fname):
-        return fname.endswith('.vala') or fname.endswith('.vapi')
+    def can_compile(self, filename):
+        suffix = filename.split('.')[-1]
+        return suffix in ('vala', 'vapi')
 
 class RustCompiler():
     def __init__(self, exelist, version):
@@ -838,6 +856,9 @@ class RustCompiler():
     def needs_static_linker(self):
         return False
 
+    def name_string(self):
+        return ' '.join(self.exelist)
+
     def get_exelist(self):
         return self.exelist
 
@@ -1013,7 +1034,7 @@ def get_gcc_soname_args(gcc_type, shlib_name, path, soversion):
     elif gcc_type == GCC_OSX:
         return ['-install_name', os.path.join(path, 'lib' + shlib_name + '.dylib')]
     else:
-        raise RuntimeError('Not impelented yet.')
+        raise RuntimeError('Not implemented yet.')
 
 class GnuCCompiler(CCompiler):
     std_warn_args = []
@@ -1059,6 +1080,9 @@ class GnuObjCCompiler(ObjCCompiler):
     def __init__(self, exelist, version, is_cross, exe_wrapper=None):
         ObjCCompiler.__init__(self, exelist, version, is_cross, exe_wrapper)
         self.id = 'gcc'
+        # Not really correct, but GNU objc is only used on non-OSX non-win. File a bug
+        # if this breaks your use case.
+        self.gcc_type = GCC_STANDARD
 
     def get_std_warn_args(self):
         return GnuObjCCompiler.std_warn_args
@@ -1082,6 +1106,9 @@ class GnuObjCPPCompiler(ObjCPPCompiler):
     def __init__(self, exelist, version, is_cross, exe_wrapper=None):
         ObjCCompiler.__init__(self, exelist, version, is_cross, exe_wrapper)
         self.id = 'gcc'
+        # Not really correct, but GNU objc is only used on non-OSX non-win. File a bug
+        # if this breaks your use case.
+        self.gcc_type = GCC_STANDARD
 
     def get_std_warn_args(self):
         return GnuObjCPPCompiler.std_warn_args
@@ -1206,10 +1233,16 @@ class FortranCompiler():
         self.is_cross = is_cross
         self.exe_wrapper = exe_wrapper
         self.language = 'fortran'
+        # Not really correct but I don't have Fortran compilers to test with. Sorry.
+        self.gcc_type = GCC_STANDARD
+        self.id = "IMPLEMENTATION CLASSES MUST SET THIS"
 
     def get_id(self):
         return self.id
 
+    def name_string(self):
+        return ' '.join(self.exelist)
+
     def get_exelist(self):
         return self.exelist
 
