@@ -14,8 +14,22 @@
 
 import pickle, os, uuid
 
-version = '0.24.0'
+version = '0.25.0'
 
+builtin_options = {'buildtype': True,
+                          'strip': True,
+                          'coverage': True,
+                          'pch': True,
+                          'unity': True,
+                          'prefix': True,
+                          'libdir' : True,
+                          'bindir' : True,
+                          'includedir' : True,
+                          'datadir' : True,
+                          'mandir' : True,
+                          'localedir' : True,
+                          'werror' : True,
+                         }
 # This class contains all data that must persist over multiple
 # invocations of Meson. It is roughly the same thing as
 # cmakecache.
@@ -110,7 +124,7 @@ forbidden_target_names = {'clean': None,
                           'test-valgrind': None,
                           'install': None,
                           'build.ninja': None,
-                          }
+                         }
 
 class MesonException(Exception):
     def __init__(self, *args, **kwargs):
