@@ -1,4 +1,4 @@
-# Copyright 2013-2014 The Meson development team
+# Copyright 2013-2015 The Meson development team
 
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -58,57 +58,79 @@ class Dependency():
     def need_threads(self):
         return False
 
+class InternalDependency():
+    def __init__(self, incdirs, libraries, sources):
+        super().__init__()
+        self.include_directories = incdirs
+        self.libraries = libraries
+        self.sources = sources
+
 class PkgConfigDependency(Dependency):
     pkgconfig_found = None
 
-    __libtool_pat = re.compile("dlname='([A-z0-9\.\-\+]+)'\n")
-
-
-    def __init__(self, name, kwargs):
-        required = kwargs.get('required', True)
+    def __init__(self, name, environment, kwargs):
         Dependency.__init__(self)
+        self.required = kwargs.get('required', True)
+        if 'native' in kwargs and environment.is_cross_build():
+            want_cross = not kwargs['native']
+        else:
+            want_cross = environment.is_cross_build()
         self.name = name
         if PkgConfigDependency.pkgconfig_found is None:
             self.check_pkgconfig()
 
         self.is_found = False
         if not PkgConfigDependency.pkgconfig_found:
-            if required:
+            if self.required:
                 raise DependencyException('Pkg-config not found.')
             self.cargs = []
             self.libs = []
             return
-        p = subprocess.Popen(['pkg-config', '--modversion', name], stdout=subprocess.PIPE,
-                                 stderr=subprocess.PIPE)
+        if environment.is_cross_build() and want_cross:
+            if "pkgconfig" not in environment.cross_info:
+                raise DependencyException('Pkg-config binary missing from cross file.')
+            pkgbin = environment.cross_info['pkgconfig']
+            self.type_string = 'Cross'
+        else:
+            pkgbin = 'pkg-config'
+            self.type_string = 'Native'
+
+        self.pkgbin = pkgbin
+        p = subprocess.Popen([pkgbin, '--modversion', name],
+                             stdout=subprocess.PIPE,
+                             stderr=subprocess.PIPE)
         out = p.communicate()[0]
         if p.returncode != 0:
-            if required:
-                raise DependencyException('Required dependency %s not found.' % name)
+            if self.required:
+                raise DependencyException('%s dependency %s not found.' % (self.type_string, name))
             self.modversion = 'none'
             self.cargs = []
             self.libs = []
         else:
             self.modversion = out.decode().strip()
-            mlog.log('Dependency', mlog.bold(name), 'found:', mlog.green('YES'), self.modversion)
-            version_requirement = kwargs.get('version', None)
-            if version_requirement is None:
+            mlog.log('%s dependency' % self.type_string, mlog.bold(name), 'found:',
+                     mlog.green('YES'), self.modversion)
+            self.version_requirement = kwargs.get('version', None)
+            if self.version_requirement is None:
                 self.is_found = True
             else:
-                if not isinstance(version_requirement, str):
+                if not isinstance(self.version_requirement, str):
                     raise DependencyException('Version argument must be string.')
-                self.is_found = mesonlib.version_compare(self.modversion, version_requirement)
-                if not self.is_found and required:
-                    raise DependencyException('Invalid version of a dependency, needed %s %s found %s.' % (name, version_requirement, self.modversion))
+                self.is_found = mesonlib.version_compare(self.modversion, self.version_requirement)
+                if not self.is_found and self.required:
+                    raise DependencyException(
+                        'Invalid version of a dependency, needed %s %s found %s.' %
+                        (name, self.version_requirement, self.modversion))
             if not self.is_found:
                 return
-            p = subprocess.Popen(['pkg-config', '--cflags', name], stdout=subprocess.PIPE,
+            p = subprocess.Popen([pkgbin, '--cflags', name], stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE)
             out = p.communicate()[0]
             if p.returncode != 0:
                 raise RuntimeError('Could not generate cargs for %s.' % name)
             self.cargs = out.decode().split()
 
-            p = subprocess.Popen(['pkg-config', '--libs', name], stdout=subprocess.PIPE,
+            p = subprocess.Popen([pkgbin, '--libs', name], stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE)
             out = p.communicate()[0]
             if p.returncode != 0:
@@ -116,7 +138,7 @@ class PkgConfigDependency(Dependency):
             self.libs = []
             for lib in out.decode().split():
                 if lib.endswith(".la"):
-                    shared_libname = self.__extract_libtool_shlib(lib)
+                    shared_libname = self.extract_libtool_shlib(lib)
                     shared_lib = os.path.join(os.path.dirname(lib), shared_libname)
                     if not os.path.exists(shared_lib):
                         shared_lib = os.path.join(os.path.dirname(lib), ".libs", shared_libname)
@@ -129,6 +151,20 @@ class PkgConfigDependency(Dependency):
 
                 self.libs.append(lib)
 
+    def get_variable(self, variable_name):
+        p = subprocess.Popen([self.pkgbin, '--variable=%s' % variable_name, self.name],
+                             stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        out = p.communicate()[0]
+        if p.returncode != 0:
+            if self.required:
+                raise DependencyException('%s dependency %s not found.' %
+                                          (self.type_string, self.name))
+        else:
+            variable = out.decode().strip()
+        mlog.debug('return of subprocess : %s' % variable)
+
+        return variable
+
     def get_modversion(self):
         return self.modversion
 
@@ -156,33 +192,36 @@ class PkgConfigDependency(Dependency):
     def found(self):
         return self.is_found
 
-    def __extract_dlname_field(self, la_file):
-        f = open(la_file)
-        data = f.read()
-        f.close()
-        m = self.__libtool_pat.search(data)
-        if m:
-            return m.groups()[0]
-        else:
-            return None
+    def extract_field(self, la_file, fieldname):
+        for line in open(la_file):
+            arr = line.strip().split('=')
+            if arr[0] == fieldname:
+                return arr[1][1:-1]
+        return None
 
-    def __extract_libtool_shlib(self, la_file):
+    def extract_dlname_field(self, la_file):
+        return self.extract_field(la_file, 'dlname')
+
+    def extract_libdir_field(self, la_file):
+        return self.extract_field(la_file, 'libdir')
+
+    def extract_libtool_shlib(self, la_file):
         '''
         Returns the path to the shared library
         corresponding to this .la file
         '''
-        dlname = self.__extract_dlname_field(la_file)
+        dlname = self.extract_dlname_field(la_file)
         if dlname is None:
             return None
 
         # Darwin uses absolute paths where possible; since the libtool files never
         # contain absolute paths, use the libdir field
-        if platform.system() == 'Darwin':
+        if mesonlib.is_osx():
             dlbasename = os.path.basename(dlname)
-            libdir = self._extract_libdir_field(la_file)
+            libdir = self.extract_libdir_field(la_file)
             if libdir is None:
                 return dlbasename
-            return libdir + '/' + dlbasename
+            return os.path.join(libdir, dlbasename)
         # From the comments in extract_libtool(), older libtools had
         # a path rather than the raw dlname
         return os.path.basename(dlname)
@@ -190,7 +229,7 @@ class PkgConfigDependency(Dependency):
 class WxDependency(Dependency):
     wx_found = None
 
-    def __init__(self, kwargs):
+    def __init__(self, environment, kwargs):
         Dependency.__init__(self)
         if WxDependency.wx_found is None:
             self.check_wxconfig()
@@ -198,8 +237,9 @@ class WxDependency(Dependency):
         if not WxDependency.wx_found:
             raise DependencyException('Wx-config not found.')
         self.is_found = False
-        p = subprocess.Popen([self.wxc, '--version'], stdout=subprocess.PIPE,
-                                 stderr=subprocess.PIPE)
+        p = subprocess.Popen([self.wxc, '--version'],
+                             stdout=subprocess.PIPE,
+                             stderr=subprocess.PIPE)
         out = p.communicate()[0]
         if p.returncode != 0:
             mlog.log('Dependency wxwidgets found:', mlog.red('NO'))
@@ -218,7 +258,8 @@ class WxDependency(Dependency):
             self.requested_modules = self.get_requested(kwargs)
             # wx-config seems to have a cflags as well but since it requires C++,
             # this should be good, at least for now.
-            p = subprocess.Popen([self.wxc, '--cxxflags'], stdout=subprocess.PIPE,
+            p = subprocess.Popen([self.wxc, '--cxxflags'],
+                                 stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE)
             out = p.communicate()[0]
             if p.returncode != 0:
@@ -300,7 +341,8 @@ class ExternalProgram():
                         if first_line.startswith('#!'):
                             commands = first_line[2:].split('#')[0].strip().split()
                             if mesonlib.is_windows():
-                                commands[0] = commands[0].split('/')[-1] # Windows does not have /usr/bin.
+                                # Windows does not have /usr/bin.
+                                commands[0] = commands[0].split('/')[-1]
                                 if commands[0] == 'env':
                                     commands = commands[1:]
                             self.fullpath = commands + [trial]
@@ -308,7 +350,8 @@ class ExternalProgram():
                         pass
         if not silent:
             if self.found():
-                mlog.log('Program', mlog.bold(name), 'found:', mlog.green('YES'), '(%s)' % ' '.join(self.fullpath))
+                mlog.log('Program', mlog.bold(name), 'found:', mlog.green('YES'),
+                         '(%s)' % ' '.join(self.fullpath))
             else:
                 mlog.log('Program', mlog.bold(name), 'found:', mlog.red('NO'))
 
@@ -328,7 +371,8 @@ class ExternalLibrary(Dependency):
         self.fullpath = fullpath
         if not silent:
             if self.found():
-                mlog.log('Library', mlog.bold(name), 'found:', mlog.green('YES'), '(%s)' % self.fullpath)
+                mlog.log('Library', mlog.bold(name), 'found:', mlog.green('YES'),
+                         '(%s)' % self.fullpath)
             else:
                 mlog.log('Library', mlog.bold(name), 'found:', mlog.red('NO'))
 
@@ -346,7 +390,7 @@ class BoostDependency(Dependency):
     # between the two.
     name2lib = {'test' : 'unit_test_framework'}
 
-    def __init__(self, kwargs):
+    def __init__(self, environment, kwargs):
         Dependency.__init__(self)
         self.name = 'boost'
         try:
@@ -476,7 +520,7 @@ class BoostDependency(Dependency):
         return 'thread' in self.requested_modules
 
 class GTestDependency(Dependency):
-    def __init__(self, kwargs):
+    def __init__(self, environment, kwargs):
         Dependency.__init__(self)
         self.main = kwargs.get('main', False)
         self.name = 'gtest'
@@ -485,8 +529,10 @@ class GTestDependency(Dependency):
         self.include_dir = '/usr/include'
         self.src_include_dir = '/usr/src/gtest'
         self.src_dir = '/usr/src/gtest/src'
-        self.all_src = mesonlib.File.from_absolute_file(os.path.join(self.src_dir, 'gtest-all.cc'))
-        self.main_src = mesonlib.File.from_absolute_file(os.path.join(self.src_dir, 'gtest_main.cc'))
+        self.all_src = mesonlib.File.from_absolute_file(
+            os.path.join(self.src_dir, 'gtest-all.cc'))
+        self.main_src = mesonlib.File.from_absolute_file(
+            os.path.join(self.src_dir, 'gtest_main.cc'))
         self.detect()
 
     def found(self):
@@ -541,7 +587,7 @@ class GTestDependency(Dependency):
         return True
 
 class GMockDependency(Dependency):
-    def __init__(self, kwargs):
+    def __init__(self, environment, kwargs):
         Dependency.__init__(self)
         # GMock may be a library or just source.
         # Work with both.
@@ -595,35 +641,42 @@ class GMockDependency(Dependency):
         return self.is_found
 
 class Qt5Dependency(Dependency):
-    def __init__(self, kwargs):
+    def __init__(self, environment, kwargs):
         Dependency.__init__(self)
         self.name = 'qt5'
         self.root = '/usr'
         mods = kwargs.get('modules', [])
         self.cargs = []
-        self.largs= []
+        self.largs = []
         self.is_found = False
         if isinstance(mods, str):
             mods = [mods]
         if len(mods) == 0:
             raise DependencyException('No Qt5 modules specified.')
-        if shutil.which('pkg-config') is not None:
-            self.pkgconfig_detect(mods, kwargs)
+        type_text = 'native'
+        if environment.is_cross_build() and kwargs.get('native', False):
+            type_text = 'cross'
+            self.pkgconfig_detect(mods, environment, kwargs)
+        elif not environment.is_cross_build() and shutil.which('pkg-config') is not None:
+            self.pkgconfig_detect(mods, environment, kwargs)
         elif shutil.which('qmake') is not None:
             self.qmake_detect(mods, kwargs)
-        if not self.is_found:
-            mlog.log('Qt5 dependency found: ', mlog.red('NO'))
         else:
-            mlog.log('Qt5 dependency found: ', mlog.green('YES'))
+            self.version = 'none'
+        if not self.is_found:
+            mlog.log('Qt5 %s dependency found: ' % type_text, mlog.red('NO'))
+        else:
+            mlog.log('Qt5 %s dependency found: ' % type_text, mlog.green('YES'))
 
-    def pkgconfig_detect(self, mods, kwargs):
+    def pkgconfig_detect(self, mods, environment, kwargs):
         modules = []
         for module in mods:
-            modules.append(PkgConfigDependency('Qt5' + module, kwargs))
+            modules.append(PkgConfigDependency('Qt5' + module, environment, kwargs))
         for m in modules:
             self.cargs += m.get_compile_args()
             self.largs += m.get_link_args()
         self.is_found = True
+        self.version = modules[0].modversion
 
     def qmake_detect(self, mods, kwargs):
         pc = subprocess.Popen(['qmake', '-v'], stdout=subprocess.PIPE,
@@ -635,6 +688,7 @@ class Qt5Dependency(Dependency):
         if not 'version 5' in stdo:
             mlog.log('QMake is not for Qt5.')
             return
+        self.version = re.search('5(\.\d+)+', stdo).match(0)
         (stdo, _) = subprocess.Popen(['qmake', '-query'], stdout=subprocess.PIPE).communicate()
         qvars = {}
         for line in stdo.decode().split('\n'):
@@ -698,7 +752,7 @@ class Qt5Dependency(Dependency):
         return ['-fPIC']
 
 class Qt4Dependency(Dependency):
-    def __init__(self, kwargs):
+    def __init__(self, environment, kwargs):
         Dependency.__init__(self)
         self.name = 'qt4'
         self.root = '/usr'
@@ -707,7 +761,7 @@ class Qt4Dependency(Dependency):
         if isinstance(mods, str):
             mods = [mods]
         for module in mods:
-            self.modules.append(PkgConfigDependency('Qt' + module, kwargs))
+            self.modules.append(PkgConfigDependency('Qt' + module, environment, kwargs))
         if len(self.modules) == 0:
             raise DependencyException('No Qt4 modules specified.')
 
@@ -736,7 +790,7 @@ class Qt4Dependency(Dependency):
         return True
 
 class GnuStepDependency(Dependency):
-    def __init__(self, kwargs):
+    def __init__(self, environment, kwargs):
         Dependency.__init__(self)
         self.modules = kwargs.get('modules', [])
         self.detect()
@@ -760,7 +814,7 @@ class GnuStepDependency(Dependency):
         else:
             arg = '--base-libs'
         fp = subprocess.Popen([confprog, '--objc-flags'],
-                             stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+                              stdout=subprocess.PIPE, stderr=subprocess.PIPE)
         (flagtxt, flagerr) = fp.communicate()
         flagtxt = flagtxt.decode()
         flagerr = flagerr.decode()
@@ -769,7 +823,7 @@ class GnuStepDependency(Dependency):
         args = flagtxt.split()
         self.args = self.filter_arsg(args)
         fp = subprocess.Popen([confprog, arg],
-                             stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+                              stdout=subprocess.PIPE, stderr=subprocess.PIPE)
         (libtxt, liberr) = fp.communicate()
         libtxt = libtxt.decode()
         liberr = liberr.decode()
@@ -808,7 +862,7 @@ why. As a hack filter out everything that is not a flag."""
         return self.libs
 
 class AppleFrameworks(Dependency):
-    def __init__(self, kwargs):
+    def __init__(self, environment, kwargs):
         Dependency.__init__(self)
         modules = kwargs.get('modules', [])
         if isinstance(modules, str):
@@ -828,17 +882,18 @@ class AppleFrameworks(Dependency):
         return mesonlib.is_osx()
 
 class GLDependency(Dependency):
-    def __init__(self, kwargs):
+    def __init__(self, environment, kwargs):
         Dependency.__init__(self)
         self.is_found = False
         self.cargs = []
         self.linkargs = []
         try:
-            pcdep = PkgConfigDependency('gl', kwargs)
+            pcdep = PkgConfigDependency('gl', environment, kwargs)
             if pcdep.found():
                 self.is_found = True
                 self.cargs = pcdep.get_compile_args()
                 self.linkargs = pcdep.get_link_args()
+                return
         except Exception:
             pass
         if mesonlib.is_osx():
@@ -855,17 +910,21 @@ class GLDependency(Dependency):
 # There are three different ways of depending on SDL2:
 # sdl2-config, pkg-config and OSX framework
 class SDL2Dependency(Dependency):
-    def __init__(self, kwargs):
+    def __init__(self, environment, kwargs):
         Dependency.__init__(self)
         self.is_found = False
         self.cargs = []
         self.linkargs = []
         sdlconf = shutil.which('sdl2-config')
         if sdlconf:
-            pc = subprocess.Popen(['sdl2-config', '--cflags'], stdout=subprocess.PIPE, stderr=subprocess.DEVNULL)
+            pc = subprocess.Popen(['sdl2-config', '--cflags'],
+                                  stdout=subprocess.PIPE,
+                                  stderr=subprocess.DEVNULL)
             (stdo, _) = pc.communicate()
             self.cargs = stdo.decode().strip().split()
-            pc = subprocess.Popen(['sdl2-config', '--libs'], stdout=subprocess.PIPE, stderr=subprocess.DEVNULL)
+            pc = subprocess.Popen(['sdl2-config', '--libs'],
+                                  stdout=subprocess.PIPE,
+                                  stderr=subprocess.DEVNULL)
             (stdo, _) = pc.communicate()
             self.linkargs = stdo.decode().strip().split()
             self.is_found = True
@@ -904,7 +963,8 @@ class ExtraFrameworkDependency(Dependency):
         self.name = None
         self.detect(name, path)
         if self.found():
-            mlog.log('Dependency', mlog.bold(name), 'found:', mlog.green('YES'), os.path.join(self.path, self.name))
+            mlog.log('Dependency', mlog.bold(name), 'found:', mlog.green('YES'),
+                     os.path.join(self.path, self.name))
         else:
             mlog.log('Dependency', name, 'found:', mlog.red('NO'))
 
@@ -945,22 +1005,22 @@ def get_dep_identifier(name, kwargs):
         modlist = [modlist]
     for module in modlist:
         elements.append(module)
-    return '/'.join(elements)
+    return '/'.join(elements) + '/main' + str(kwargs.get('main', False))
 
-def find_external_dependency(name, kwargs):
+def find_external_dependency(name, environment, kwargs):
     required = kwargs.get('required', True)
     if not isinstance(required, bool):
         raise DependencyException('Keyword "required" must be a boolean.')
     lname = name.lower()
     if lname in packages:
-        dep = packages[lname](kwargs)
+        dep = packages[lname](environment, kwargs)
         if required and not dep.found():
             raise DependencyException('Dependency "%s" not found' % name)
         return dep
     pkg_exc = None
     pkgdep = None
     try:
-        pkgdep = PkgConfigDependency(name, kwargs)
+        pkgdep = PkgConfigDependency(name, environment, kwargs)
         if pkgdep.found():
             return pkgdep
     except Exception as e:
@@ -987,4 +1047,4 @@ packages = {'boost': BoostDependency,
             'wxwidgets' : WxDependency,
             'sdl2' : SDL2Dependency,
             'gl' : GLDependency,
-            }
+           }
