@@ -22,7 +22,6 @@ import optinterpreter
 import wrap
 import mesonlib
 import os, sys, platform, subprocess, shutil, uuid, re
-from mesonlib import File
 from functools import wraps
 
 import importlib
@@ -44,6 +43,14 @@ def check_stringlist(a, msg='Arguments must be strings.'):
     if not all(isinstance(s, str) for s in a):
         raise InvalidArguments(msg)
 
+def noPosargs(f):
+    @wraps(f)
+    def wrapped(self, node, args, kwargs):
+        if len(args) != 0:
+            raise InvalidArguments('Function does not take positional arguments.')
+        return f(self, node, args, kwargs)
+    return wrapped
+
 def noKwargs(f):
     @wraps(f)
     def wrapped(self, node, args, kwargs):
@@ -86,7 +93,7 @@ class TryRunResultHolder(InterpreterObject):
                              'compiled' : self.compiled_method,
                              'stdout' : self.stdout_method,
                              'stderr' : self.stderr_method,
-                             })
+                            })
 
     def returncode_method(self, args, kwargs):
         return self.res.returncode
@@ -112,7 +119,7 @@ class RunProcess(InterpreterObject):
         self.methods.update({'returncode' : self.returncode_method,
                              'stdout' : self.stdout_method,
                              'stderr' : self.stderr_method,
-                             })
+                            })
 
     def run_command(self, command_array, source_dir, build_dir, subdir, in_builddir):
         cmd_name = command_array[0]
@@ -167,7 +174,7 @@ class ConfigurationDataHolder(InterpreterObject):
         self.held_object = build.ConfigurationData()
         self.methods.update({'set': self.set_method,
                              'set10': self.set10_method,
-                             })
+                            })
 
     def is_used(self):
         return self.used
@@ -215,6 +222,15 @@ class DependencyHolder(InterpreterObject):
     def found_method(self, args, kwargs):
         return self.held_object.found()
 
+class InternalDependencyHolder(InterpreterObject):
+    def __init__(self, dep):
+        InterpreterObject.__init__(self)
+        self.held_object = dep
+        self.methods.update({'found' : self.found_method})
+
+    def found_method(self, args, kwargs):
+        return True
+
 class ExternalProgramHolder(InterpreterObject):
     def __init__(self, ep):
         InterpreterObject.__init__(self)
@@ -290,7 +306,7 @@ class Build(InterpreterObject):
     def __init__(self):
         InterpreterObject.__init__(self)
         self.methods.update({'name' : self.get_name_method,
-                             })
+                            })
 
     def get_name_method(self, args, kwargs):
         return platform.system().lower()
@@ -303,7 +319,7 @@ class Host(InterpreterObject):
         self.environment = envir
         self.methods.update({'name' : self.get_name_method,
                              'is_big_endian' : self.is_big_endian_method,
-                             })
+                            })
 
     def get_name_method(self, args, kwargs):
         if self.environment.is_cross_build():
@@ -360,9 +376,6 @@ class Data(InterpreterObject):
         if not isinstance(self.install_dir, str):
             raise InterpreterException('Custom_install_dir must be a string.')
 
-    def get_install_subdir(self):
-        return self.install_subdir
-
     def get_source_subdir(self):
         return self.source_subdir
 
@@ -478,9 +491,8 @@ class SubprojectHolder(InterpreterObject):
     def __init__(self, subinterpreter):
         super().__init__()
         self.subinterpreter = subinterpreter
-        self.methods.update({
-                             'get_variable' : self.get_variable_method,
-                             })
+        self.methods.update({'get_variable' : self.get_variable_method,
+                            })
 
     def get_variable_method(self, args, kwargs):
         if len(args) != 1:
@@ -506,7 +518,7 @@ class CompilerHolder(InterpreterObject):
                              'alignment' : self.alignment_method,
                              'version' : self.version_method,
                              'cmd_array' : self.cmd_array_method,
-                             })
+                            })
 
     def version_method(self, args, kwargs):
         return self.compiler.version
@@ -654,7 +666,7 @@ class ModuleHolder(InterpreterObject):
             raise InvalidArguments('Module %s does not have method %s.' % (self.modname, method_name))
         state = ModuleState()
         state.build_to_src = os.path.relpath(self.interpreter.environment.get_source_dir(),
-                                            self.interpreter.environment.get_build_dir())
+                                             self.interpreter.environment.get_build_dir())
         state.subdir = self.interpreter.subdir
         state.environment = self.interpreter.environment
         state.project_name = self.interpreter.build.project_name
@@ -679,8 +691,10 @@ class MesonMain(InterpreterObject):
                              'is_subproject' : self.is_subproject_method,
                              'current_source_dir' : self.current_source_dir_method,
                              'current_build_dir' : self.current_build_dir_method,
+                             'source_root' : self.source_root_method,
+                             'build_root' : self.build_root_method,
                              'set_install_script' : self.set_install_script_method,
-                             })
+                            })
 
     def set_install_script_method(self, args, kwargs):
         if len(args) != 1:
@@ -707,6 +721,12 @@ class MesonMain(InterpreterObject):
             return src
         return os.path.join(src, sub)
 
+    def source_root_method(self, args, kwargs):
+        return self.interpreter.environment.source_dir
+
+    def build_root_method(self, args, kwargs):
+        return self.interpreter.environment.build_dir
+
     def has_exe_wrapper_method(self, args, kwargs):
         if self.is_cross_build_method(None, None):
             return 'exe_wrap' in  self.build.environment.cross_info
@@ -793,6 +813,7 @@ class Interpreter():
                       'static_library' : self.func_static_lib,
                       'shared_library' : self.func_shared_lib,
                       'jar' : self.func_jar,
+                      'build_target': self.func_build_target,
                       'custom_target' : self.func_custom_target,
                       'run_target' : self.func_run_target,
                       'generator' : self.func_generator,
@@ -818,7 +839,8 @@ class Interpreter():
                       'set_variable' : self.func_set_variable,
                       'import' : self.func_import,
                       'files' : self.func_files,
-                      }
+                      'declare_dependency': self.func_declare_dependency,
+                     }
 
     def module_method_callback(self, invalues):
         unwrap_single = False
@@ -921,7 +943,22 @@ class Interpreter():
     @stringArgs
     @noKwargs
     def func_files(self, node, args, kwargs):
-        return [File.from_source_file(self.environment.source_dir, self.subdir, fname) for fname in args]
+        return [mesonlib.File.from_source_file(self.environment.source_dir, self.subdir, fname) for fname in args]
+
+    @noPosargs
+    def func_declare_dependency(self, node, args, kwargs):
+        incs = kwargs.get('include_directories', [])
+        if not isinstance(incs, list):
+            incs = [incs]
+        libs = kwargs.get('link_with', [])
+        if not isinstance(libs, list):
+            libs = [libs]
+        sources = kwargs.get('sources', [])
+        if not isinstance(sources, list):
+            sources = [sources]
+        sources = self.source_strings_to_files(self.flatten(sources))
+        dep = dependencies.InternalDependency(incs, libs, sources)
+        return InternalDependencyHolder(dep)
 
     def set_variable(self, varname, variable):
         if variable is None:
@@ -977,8 +1014,8 @@ class Interpreter():
     def validate_arguments(self, args, argcount, arg_types):
         if argcount is not None:
             if argcount != len(args):
-                raise InvalidArguments('Expected %d arguments, got %d',
-                                       argcount, len(args))
+                raise InvalidArguments('Expected %d arguments, got %d.' %
+                                       (argcount, len(args)))
         for i in range(min(len(args), len(arg_types))):
             wanted = arg_types[i]
             actual = args[i]
@@ -1093,7 +1130,7 @@ class Interpreter():
         if len(args) != 1:
             raise InterpreterException('Argument required for get_option.')
         optname = args[0]
-        if self.is_subproject():
+        if optname not in coredata.builtin_options and self.is_subproject():
             optname = self.subproject + ':' + optname
         try:
             return self.environment.get_coredata().get_builtin_option(optname)
@@ -1273,7 +1310,7 @@ class Interpreter():
         else:
             dep = dependencies.Dependency() # Returns always false for dep.found()
         if not dep.found():
-            dep = dependencies.find_external_dependency(name, kwargs)
+            dep = dependencies.find_external_dependency(name, self.environment, kwargs)
         self.coredata.deps[identifier] = dep
         return DependencyHolder(dep)
 
@@ -1289,6 +1326,21 @@ class Interpreter():
     def func_jar(self, node, args, kwargs):
         return self.build_target(node, args, kwargs, JarHolder)
 
+    def func_build_target(self, node, args, kwargs):
+        if 'target_type' not in kwargs:
+            raise InterpreterException('Missing target_type keyword argument')
+        target_type = kwargs.pop('target_type')
+        if target_type == 'executable':
+            return self.func_executable(node, args, kwargs)
+        elif target_type == 'shared_library':
+            return self.func_shared_lib(node, args, kwargs)
+        elif target_type == 'static_library':
+            return self.func_static_lib(node, args, kwargs)
+        elif target_type == 'jar':
+            return self.func_jar(node, args, kwargs)
+        else:
+            raise InterpreterException('Unknown target_type.')
+
     def func_vcs_tag(self, node, args, kwargs):
         fallback = kwargs.pop('fallback', None)
         if not isinstance(fallback, str):
@@ -1321,13 +1373,8 @@ class Interpreter():
         if len(args) != 1:
             raise InterpreterException('Incorrect number of arguments')
         name = args[0]
-        if name in coredata.forbidden_target_names:
-            raise InvalidArguments('Target name "%s" is reserved for Meson\'s internal use. Please rename.'\
-                                   % name)
-        if name in self.build.targets:
-            raise InvalidCode('Tried to create target "%s", but a target of that name already exists.' % name)
         tg = CustomTargetHolder(build.CustomTarget(name, self.subdir, kwargs))
-        self.build.targets[name] = tg.held_object
+        self.add_target(name, tg.held_object)
         return tg
 
     @stringArgs
@@ -1336,15 +1383,10 @@ class Interpreter():
         if len(args) < 2:
             raise InterpreterException('Incorrect number of arguments')
         name = args[0]
-        if name in coredata.forbidden_target_names:
-            raise InvalidArguments('Target name "%s" is reserved for Meson\'s internal use. Please rename.'\
-                                   % name)
-        if name in self.build.targets:
-            raise InvalidCode('Tried to create target "%s", but a target of that name already exists.' % name)
         command = args[1]
         cmd_args = args[2:]
         tg = RunTargetHolder(name, command, cmd_args, self.subdir)
-        self.build.targets[name] = tg.held_object
+        self.add_target(name, tg.held_object)
         return tg
 
     def func_generator(self, node, args, kwargs):
@@ -1547,16 +1589,29 @@ class Interpreter():
     def source_strings_to_files(self, sources):
         results = []
         for s in sources:
-            if isinstance(s, File) or isinstance(s, GeneratedListHolder) or \
+            if isinstance(s, mesonlib.File) or isinstance(s, GeneratedListHolder) or \
             isinstance(s, CustomTargetHolder):
                 pass
             elif isinstance(s, str):
-                s = File.from_source_file(self.environment.source_dir, self.subdir, s)
+                s = mesonlib.File.from_source_file(self.environment.source_dir, self.subdir, s)
             else:
-                raise RuntimeError("Unreachable code")
+                raise InterpreterException("Source item is not string or File-type object.")
             results.append(s)
         return results
 
+    def add_target(self, name, tobj):
+        if name in coredata.forbidden_target_names:
+            raise InvalidArguments('Target name "%s" is reserved for Meson\'s internal use. Please rename.'\
+                                   % name)
+        # To permit an executable and a shared library to have the
+        # same name, such as "foo.exe" and "libfoo.a".
+        idname = name + tobj.type_suffix()
+        if idname in self.build.targets:
+            raise InvalidCode('Tried to create target "%s", but a target of that name already exists.' % name)
+        self.build.targets[idname] = tobj
+        if idname not in self.coredata.target_guids:
+            self.coredata.target_guids[idname] = str(uuid.uuid4()).upper()
+
     def build_target(self, node, args, kwargs, targetholder):
         name = args[0]
         sources = args[1:]
@@ -1567,9 +1622,6 @@ class Interpreter():
                 is_cross = True
         else:
             is_cross = False
-        if name in coredata.forbidden_target_names:
-            raise InvalidArguments('Target name "%s" is reserved for Meson\'s internal use. Please rename.'\
-                                   % name)
         try:
             kw_src = self.flatten(kwargs['sources'])
             if not isinstance(kw_src, list):
@@ -1579,10 +1631,9 @@ class Interpreter():
         sources += kw_src
         sources = self.source_strings_to_files(sources)
         objs = self.flatten(kwargs.get('objects', []))
+        kwargs['dependencies'] = self.flatten(kwargs.get('dependencies', []))
         if not isinstance(objs, list):
             objs = [objs]
-        if name in self.build.targets:
-            raise InvalidCode('Tried to create target "%s", but a target of that name already exists.' % name)
         self.check_sources_exist(os.path.join(self.source_root, self.subdir), sources)
         if targetholder is ExecutableHolder:
             targetclass = build.Executable
@@ -1593,13 +1644,11 @@ class Interpreter():
         elif targetholder is JarHolder:
             targetclass = build.Jar
         else:
-            print(targetholder)
+            mlog.debug('Unknown target type:', str(targetholder))
             raise RuntimeError('Unreachable code')
         target = targetclass(name, self.subdir, is_cross, sources, objs, self.environment, kwargs)
         l = targetholder(target)
-        self.build.targets[name] = l.held_object
-        if name not in self.coredata.target_guids:
-            self.coredata.target_guids[name] = str(uuid.uuid4()).upper()
+        self.add_target(name, l.held_object)
         self.global_args_frozen = True
         return l
 
@@ -1625,7 +1674,7 @@ class Interpreter():
             isinstance(value, str) or\
             isinstance(value, int) or \
             isinstance(value, list) or \
-            isinstance(value, File):
+            isinstance(value, mesonlib.File):
             return True
         return False
 
@@ -1713,8 +1762,21 @@ class Interpreter():
         if not isinstance(obj, InterpreterObject):
             raise InvalidArguments('Variable "%s" is not callable.' % object_name)
         (args, kwargs) = self.reduce_arguments(args)
+        if method_name == 'extract_objects':
+            self.validate_extraction(obj.held_object)
         return obj.method_call(method_name, args, kwargs)
 
+    # Only permit object extraction from the same subproject
+    def validate_extraction(self, buildtarget):
+        if not self.subdir.startswith(self.subproject_dir):
+            if buildtarget.subdir.startswith(self.subproject_dir):
+                raise InterpreterException('Tried to extract objects from a subproject target.')
+        else:
+            if not buildtarget.subdir.startswith(self.subproject_dir):
+                raise InterpreterException('Tried to extract objects from the main project from a subproject.')
+            if self.subdir.split('/')[1] != buildtarget.subdir.split('/')[1]:
+                raise InterpreterException('Tried to extract objects from a different subproject.')
+
     def array_method_call(self, obj, method_name, args):
         if method_name == 'contains':
             return self.check_contains(obj, args)
