@@ -55,6 +55,7 @@ class GnomeModule:
         nsversion = kwargs.pop('nsversion')
         libsources = kwargs.pop('sources')
         girfile = '%s-%s.gir' % (ns, nsversion)
+        depends = [girtarget]
 
         scan_command = ['g-ir-scanner', '@INPUT@']
         scan_command += pkgargs
@@ -65,6 +66,17 @@ class GnomeModule:
             for incdir in incdirs.get_incdirs():
                 scan_command += ['-I%s' % os.path.join(state.environment.get_source_dir(), incdir)]
 
+        if 'link_with' in kwargs:
+            link_with = kwargs.pop('link_with')
+            for link in link_with:
+                lib = link.held_object
+                scan_command += ['-l%s' % lib.name]
+                if isinstance(lib, build.SharedLibrary):
+                    scan_command += ['-L%s' %
+                            os.path.join(state.environment.get_build_dir(),
+                                lib.subdir)]
+                    depends.append(lib)
+
         if 'includes' in kwargs:
             includes = kwargs.pop('includes')
             if isinstance(includes, str):
@@ -95,6 +107,17 @@ class GnomeModule:
                 scan_command += ['--pkg-export=%s' % pkg for pkg in pkgs]
             else:
                 raise MesonException('Gir export packages must be str or list')
+
+        deps = None
+        if 'dependencies' in kwargs:
+            deps = kwargs.pop('dependencies')
+            if not isinstance (deps, list):
+                deps = [deps]
+            for dep in deps:
+                girdir = dep.held_object.get_variable ("girdir")
+                if girdir:
+                    scan_command += ["--add-include-path=%s" % girdir]
+
         inc_dirs = None
         if kwargs.get('include_directories'):
             inc_dirs = kwargs.pop('include_directories')
@@ -105,13 +128,14 @@ class GnomeModule:
         if isinstance(girtarget, build.Executable):
             scan_command += ['--program', girtarget]
         elif isinstance(girtarget, build.SharedLibrary):
+            scan_command += ["-L", os.path.join (state.environment.get_build_dir(), girtarget.subdir)]
             libname = girtarget.get_basename()
-            if girtarget.soversion:
-                libname += "-%s" % girtarget.soversion
             scan_command += ['--library', libname]
         scankwargs = {'output' : girfile,
                       'input' : libsources,
-                      'command' : scan_command}
+                      'command' : scan_command,
+                      'depends' : depends,
+                     }
         if kwargs.get('install'):
             scankwargs['install'] = kwargs['install']
             scankwargs['install_dir'] = os.path.join(state.environment.get_datadir(), 'gir-1.0')
@@ -120,7 +144,14 @@ class GnomeModule:
         typelib_output = '%s-%s.typelib' % (ns, nsversion)
         typelib_cmd = ['g-ir-compiler', scan_target, '--output', '@OUTPUT@']
         if inc_dirs:
-            typelib_cmd += ['--includedir=%s' % inc for inc in inc_dirs.held_object.get_incdirs()]
+            typelib_cmd += ['--includedir=%s' % inc for inc in
+                    inc_dirs.held_object.get_incdirs()]
+        if deps:
+            for dep in deps:
+                girdir = dep.held_object.get_variable ("girdir")
+                if girdir:
+                    typelib_cmd += ["--includedir=%s" % girdir]
+
         kwargs['output'] = typelib_output
         kwargs['command'] = typelib_cmd
         # Note that this can't be libdir, because e.g. on Debian it points to
