@@ -164,7 +164,7 @@ class NinjaBackend(backends.Backend):
             self.generate_custom_target(target, outfile)
         if isinstance(target, build.RunTarget):
             self.generate_run_target(target, outfile)
-        name = target.get_basename()
+        name = target.get_basename() + target.type_suffix()
         gen_src_deps = []
         if name in self.processed_targets:
             return
@@ -217,7 +217,7 @@ class NinjaBackend(backends.Backend):
             else:
                 for src in gensource.get_outfilelist():
                     if self.environment.is_object(src):
-                        obj_list.append(os.path.join(self.get_target_dir(target), target.get_basename() + '.dir', src))
+                        obj_list.append(os.path.join(self.get_target_private_dir(target), src))
                     elif not self.environment.is_header(src):
                         if is_unity:
                             if '/' in src:
@@ -266,7 +266,7 @@ class NinjaBackend(backends.Backend):
 
     def process_target_dependencies(self, target, outfile):
         for t in target.get_dependencies():
-            tname = t.get_basename()
+            tname = t.get_basename() + t.type_suffix()
             if not tname in self.processed_targets:
                 self.generate_target(t, outfile)
 
@@ -305,7 +305,7 @@ class NinjaBackend(backends.Backend):
         elem.add_item('COMMAND', cmd)
         elem.add_item('description', 'Generating %s with a custom command.' % target.name)
         elem.write(outfile)
-        self.processed_targets[target.name] = True
+        self.processed_targets[target.name + target.type_suffix()] = True
 
     def generate_run_target(self, target, outfile):
         runnerscript = os.path.join(self.environment.get_script_dir(), 'commandrunner.py')
@@ -316,7 +316,7 @@ class NinjaBackend(backends.Backend):
         elem.add_item('description', 'Running external command %s.' % target.name)
         elem.add_item('pool', 'console')
         elem.write(outfile)
-        self.processed_targets[target.name] = True
+        self.processed_targets[target.name + target.type_suffix()] = True
 
     def generate_po(self, outfile):
         for p in self.build.pot:
@@ -590,7 +590,7 @@ class NinjaBackend(backends.Backend):
                 a = '-resource:' + rel_sourcefile
             elif r.endswith('.txt') or r.endswith('.resx'):
                 ofilebase = os.path.splitext(os.path.basename(r))[0] + '.resources'
-                ofilename = os.path.join(self.get_target_dir(target), target.get_basename() + '.dir', ofilebase)
+                ofilename = os.path.join(self.get_target_private_dir(target), ofilebase)
                 elem = NinjaBuildElement(ofilename, "CUSTOM_COMMAND", rel_sourcefile)
                 elem.add_item('COMMAND', ['resgen', rel_sourcefile, ofilename])
                 elem.add_item('DESC', 'Compiling resource %s.' % rel_sourcefile)
@@ -643,8 +643,7 @@ class NinjaBackend(backends.Backend):
         rel_src = src.rel_to_builddir(self.build_to_src)
         plain_class_path = src.fname[:-4] + 'class'
         rel_obj = os.path.join(self.get_target_private_dir(target), plain_class_path)
-        element = NinjaBuildElement(rel_obj,
-                    compiler.get_language() + '_COMPILER', rel_src)
+        element = NinjaBuildElement(rel_obj, compiler.get_language() + '_COMPILER', rel_src)
         element.add_item('ARGS', args)
         element.write(outfile)
         return plain_class_path
@@ -664,7 +663,7 @@ class NinjaBackend(backends.Backend):
             if not s.endswith('.vala'):
                 continue
             vapibase = os.path.basename(s.fname)[:-4] + 'vapi'
-            rel_vapi = os.path.join(self.get_target_dir(target), target.get_basename() + '.dir', vapibase)
+            rel_vapi = os.path.join(self.get_target_private_dir(target), vapibase)
             args = ['--fast-vapi=' + rel_vapi]
             rel_s = s.rel_to_builddir(self.build_to_src)
             element = NinjaBuildElement(rel_vapi, valac.get_language() + '_COMPILER', rel_s)
@@ -690,6 +689,7 @@ class NinjaBackend(backends.Backend):
         generated_c = []
         (src, vapi_src) = self.split_vala_sources(target.get_sources())
         vapi_src = [x.rel_to_builddir(self.build_to_src) for x in vapi_src]
+        extra_dep_files = []
         for s in src:
             if not s.endswith('.vala'):
                 continue
@@ -703,19 +703,33 @@ class NinjaBackend(backends.Backend):
                 (vapibase, rel_vapi) = vapi_info
                 args += ['--use-fast-vapi=' + rel_vapi]
                 vapi_order_deps.append(rel_vapi)
-            relsc = os.path.join(self.get_target_dir(target), target.get_basename() + '.dir', sc)
+            relsc = os.path.join(self.get_target_private_dir(target), sc)
             rel_s = s.rel_to_builddir(self.build_to_src)
             args += ['--deps', relsc + '.d']
             if self.environment.coredata.werror:
                 args += valac.get_werror_args()
             for d in target.external_deps:
                 if isinstance(d, dependencies.PkgConfigDependency):
+                    if d.name == 'glib-2.0' and d.version_requirement is not None \
+                        and d.version_requirement.startswith(('>=', '==')):
+                            args += ['--target-glib', d.version_requirement[2:]]
                     args += ['--pkg', d.name]
             args += vapi_src
+            extra_args = []
+
+            for a in target.extra_args.get('vala', []):
+                if isinstance(a, File):
+                    relname = a.rel_to_builddir(self.build_to_src)
+                    extra_dep_files.append(relname)
+                    extra_args.append(relname)
+                else:
+                    extra_args.append(a)
+            args += extra_args
             generated_c += [relsc]
             element = NinjaBuildElement(relsc, valac.get_language() + '_COMPILER', rel_s)
             element.add_item('ARGS', args)
             element.add_orderdep(vapi_order_deps)
+            element.add_dep(extra_dep_files)
             element.write(outfile)
         return generated_c
 
@@ -772,7 +786,7 @@ class NinjaBackend(backends.Backend):
         rule = 'rule STATIC%s_LINKER\n' % crstr
         command = ' command = %s  $LINK_ARGS %s $in\n' % \
         (' '.join(static_linker.get_exelist()),
-        ' '.join(static_linker.get_output_args('$out')))
+         ' '.join(static_linker.get_output_args('$out')))
         description = ' description = Static linking library $out\n\n'
         outfile.write(rule)
         outfile.write(command)
@@ -801,7 +815,7 @@ class NinjaBackend(backends.Backend):
         scriptdir = self.environment.get_script_dir()
         outfile.write('\n')
         symrule = 'rule SHSYM\n'
-        symcmd = ' command = "%s" "%s" "%s" "%s" $CROSS\n' % (ninja_quote(sys.executable),
+        symcmd = ' command = "%s" "%s" %s %s $CROSS\n' % (ninja_quote(sys.executable),
                                          ninja_quote(os.path.join(scriptdir, 'symbolextractor.py')),
                                          '$in', '$out')
         synstat = ' restat = 1\n'
@@ -1066,7 +1080,7 @@ rule FORTRAN_DEP_HACK
     def get_fortran_deps(self, compiler, src, target):
         mod_files = []
         usere = re.compile(r"\s*use\s+(\w+)", re.IGNORECASE)
-        dirname = os.path.join(self.get_target_dir(target), target.get_basename() + '.dir')
+        dirname = self.get_target_private_dir(target)
         tdeps= self.fortran_deps[target.get_basename()]
         for line in open(src):
             usematch = usere.match(line)
@@ -1168,12 +1182,12 @@ rule FORTRAN_DEP_HACK
             # Dependency hack. Remove once multiple outputs in Ninja is fixed:
             # https://groups.google.com/forum/#!topic/ninja-build/j-2RfBIOd_8
             for modname, srcfile in self.fortran_deps[target.get_basename()].items():
-                modfile = os.path.join(self.get_target_dir(target), target.get_basename() + '.dir',
+                modfile = os.path.join(self.get_target_private_dir(target),
                                        compiler.module_name_to_filename(modname))
                 if srcfile == src:
                     depelem = NinjaBuildElement(modfile, 'FORTRAN_DEP_HACK', rel_obj)
                     depelem.write(outfile)
-            commands += compiler.get_module_outdir_args(os.path.join(self.get_target_dir(target), target.get_basename() + '.dir'))
+            commands += compiler.get_module_outdir_args(self.get_target_private_dir(target))
 
         element = NinjaBuildElement(rel_obj, compiler_name, rel_src)
         for d in header_deps:
