@@ -16,20 +16,7 @@ import mparser
 import coredata
 import os, re
 
-forbidden_option_names = {'buildtype': True,
-                          'strip': True,
-                          'coverage': True,
-                          'pch': True,
-                          'unity': True,
-                          'prefix': True,
-                          'libdir' : True,
-                          'bindir' : True,
-                          'includedir' : True,
-                          'datadir' : True,
-                          'mandir' : True,
-                          'localedir' : True,
-                          'werror' : True,
-                          }
+forbidden_option_names = coredata.builtin_options
 
 class OptionException(coredata.MesonException):
     pass
@@ -37,16 +24,17 @@ class OptionException(coredata.MesonException):
 optname_regex = re.compile('[^a-zA-Z0-9_-]')
 
 class UserOption:
-    def __init__(self, kwargs):
+    def __init__(self, name, kwargs):
         super().__init__()
         self.description = kwargs.get('description', '')
+        self.name = name
 
     def parse_string(self, valuestring):
         return valuestring
 
 class UserStringOption(UserOption):
-    def __init__(self, kwargs):
-        super().__init__(kwargs)
+    def __init__(self, name, kwargs):
+        super().__init__(name, kwargs)
         self.set_value(kwargs.get('value', ''))
 
     def set_value(self, newvalue):
@@ -55,13 +43,13 @@ class UserStringOption(UserOption):
         self.value = newvalue
 
 class UserBooleanOption(UserOption):
-    def __init__(self, kwargs):
-        super().__init__(kwargs)
+    def __init__(self, name, kwargs):
+        super().__init__(name, kwargs)
         self.set_value(kwargs.get('value', 'true'))
 
     def set_value(self, newvalue):
         if not isinstance(newvalue, bool):
-            raise OptionException('Value "%s" for boolean option "%s" is not a boolean.' % (valuestring, self.name))
+            raise OptionException('Value "%s" for boolean option "%s" is not a boolean.' % (str(newvalue), self.name))
         self.value = newvalue
 
     def parse_string(self, valuestring):
@@ -72,8 +60,8 @@ class UserBooleanOption(UserOption):
         raise OptionException('Value "%s" for boolean option "%s" is not a boolean.' % (valuestring, self.name))
 
 class UserComboOption(UserOption):
-    def __init__(self, kwargs):
-        super().__init__(kwargs)
+    def __init__(self, name, kwargs):
+        super().__init__(name, kwargs)
         if 'choices' not in kwargs:
             raise OptionException('Combo option missing "choices" keyword.')
         self.choices = kwargs['choices']
@@ -93,7 +81,7 @@ class UserComboOption(UserOption):
 option_types = {'string' : UserStringOption,
                 'boolean' : UserBooleanOption,
                 'combo' : UserComboOption,
-                }
+               }
 
 class OptionInterpreter:
     def __init__(self, subproject, command_line_options):
@@ -124,9 +112,7 @@ class OptionInterpreter:
                 raise e
 
     def reduce_single(self, arg):
-        if isinstance(arg, mparser.IdNode):
-            return self.get_variable(arg.value)
-        elif isinstance(arg, str):
+        if isinstance(arg, str):
             return arg
         elif isinstance(arg, mparser.StringNode):
             return arg.value
@@ -175,8 +161,7 @@ class OptionInterpreter:
             raise OptionException('Option name %s is reserved.' % opt_name)
         if self.subproject != '':
             opt_name = self.subproject + ':' + opt_name
-        opt = option_types[opt_type](kwargs)
-        opt.name = opt_name
+        opt = option_types[opt_type](opt_name, kwargs)
         if opt.description == '':
             opt.description = opt_name
         if opt_name in self.cmd_line_options:
