@@ -90,13 +90,35 @@ class Resolver:
 
 
     def get_data(self, url):
-        u = urllib.request.urlopen(url)
-        data = u.read()
-        u.close()
+        blocksize = 10*1024
+        resp = urllib.request.urlopen(url)
+        dlsize = int(resp.info()['Content-Length'])
+        print('Download size:', dlsize)
+        print('Downloading: ', end='')
+        sys.stdout.flush()
+        printed_dots = 0
+        blocks = []
+        downloaded = 0
+        while True:
+            block = resp.read(blocksize)
+            if block == b'':
+                break
+            downloaded += len(block)
+            blocks.append(block)
+            ratio = int(downloaded/dlsize * 10)
+            while printed_dots < ratio:
+                print('.', end='')
+                sys.stdout.flush()
+                printed_dots += 1
+        print('')
+        resp.close()
+        return b''.join(blocks)
+
+    def get_hash(self, data):
         h = hashlib.sha256()
         h.update(data)
         hashvalue = h.hexdigest()
-        return (data, hashvalue)
+        return hashvalue
 
     def download(self, p, packagename):
         ofname = os.path.join(self.cachedir, p.get('source_filename'))
@@ -104,15 +126,17 @@ class Resolver:
             mlog.log('Using', mlog.bold(packagename), 'from cache.')
             return
         srcurl = p.get('source_url')
-        mlog.log('Dowloading', mlog.bold(packagename), 'from', srcurl)
-        (srcdata, dhash) = self.get_data(srcurl)
+        mlog.log('Dowloading', mlog.bold(packagename), 'from', mlog.bold(srcurl))
+        srcdata = self.get_data(srcurl)
+        dhash = self.get_hash(srcdata)
         expected = p.get('source_hash')
         if dhash != expected:
             raise RuntimeError('Incorrect hash for source %s:\n %s expected\n %s actual.' % (packagename, expected, dhash))
         if p.has_patch():
             purl = p.get('patch_url')
             mlog.log('Downloading patch from', mlog.bold(purl))
-            (pdata, phash) = self.get_data(purl)
+            pdata = self.get_data(purl)
+            phash = self.get_hash(pdata)
             expected = p.get('patch_hash')
             if phash != expected:
                 raise RuntimeError('Incorrect hash for patch %s:\n %s expected\n %s actual' % (packagename, expected, phash))
@@ -126,11 +150,24 @@ class Resolver:
             try:
                 import lzma
                 del lzma
-                shutil.register_unpack_format('xztar', ['.tar.xz', '.txz'], shutil._unpack_tarfile, [], "xz'ed tar-file")
+                try:
+                    shutil.register_unpack_format('xztar', ['.tar.xz', '.txz'], shutil._unpack_tarfile, [], "xz'ed tar-file")
+                except shutil.RegistryError:
+                    pass
             except ImportError:
                 pass
-        if os.path.isdir(os.path.join(self.subdir_root, package.get('directory'))):
+        target_dir = os.path.join(self.subdir_root, package.get('directory'))
+        if os.path.isdir(target_dir):
             return
-        shutil.unpack_archive(os.path.join(self.cachedir, package.get('source_filename')), self.subdir_root)
+        extract_dir = self.subdir_root
+        # Some upstreams ship packages that do not have a leading directory.
+        # Create one for them.
+        try:
+            package.get('lead_directory_missing')
+            os.mkdir(target_dir)
+            extract_dir = target_dir
+        except KeyError:
+            pass
+        shutil.unpack_archive(os.path.join(self.cachedir, package.get('source_filename')), extract_dir)
         if package.has_patch():
             shutil.unpack_archive(os.path.join(self.cachedir, package.get('patch_filename')), self.subdir_root)
